module Ruboty
  module Handlers
    class TwitterTL < Base
      on(/follow tweets by (?<name>.+)/,
        name: 'follow',
        description: 'Follow a user on twitter.')

      on(/unfollow tweets by (?<name>.+)/,
        name: 'unfollow',
        description: 'Unfollow a user on twitter.')

      on(/list tweets following\z/,
        name: 'following',
        description: 'List following users on twitter.')

      def initialize(*args)
        super

        @stream = Ruboty::TwitterTL::Stream.new(robot)
        @stream.start(users)
      end

      def follow(message)
        username = message[:name]
        userinfo = {
          message:    message.original.except(:robot),
          twitter_id: @stream.rest.user(username).id
        }

        if users.has_value?(userinfo)
          message.reply("Already following '@#{username}'.")
        else
          users.update(username => userinfo)
          @stream.update(users)
          message.reply("Followed '@#{username}'.")
        end
      rescue Twitter::Error::NotFound
        message.reply("'@#{username}' is not found.")
      rescue Twitter::Error::Forbidden
        message.reply("Unable to verify your credentials.")
      end

      def unfollow(message)
        username = message[:name]

        if userinfo = users.delete(username)
          @stream.update(users)
          message.reply("Unfollowed '@#{username}'.")
        else
          message.reply("'@#{username}' is not found.")
        end
      end

      def following(message)
        if users.empty?
          message.reply("Following no user.")
        else
          list = users.select { |_, userinfo| userinfo[:message][:from] == message.original[:from] }
          text = list.keys.map { |username| "@#{username}" }.join("\n")
          message.reply(text, code: true)
        end
      end

      def users
        robot.brain.data[Ruboty::TwitterTL::NAMESPACE] ||= {}
      end
    end
  end
end
